#include "stdafx.h"

/*
Contents : 
	CGraph members
	CVwGraphs - Graphs Display 
	VwGraphsDlgProc - message pump for options dialog 
	OpenVwGraphs( HWND hDlg, CData * ) - external entry point to open task options dialog 
*/

CItemArray g_aGraphs;

//-----------------------------------------------------------------------------
// dialog for options edit 
class CVwGraphs : public CDialog { 
	private : 
		//-- Controls 
		CDlgControl m_picGraph;
		CDlgButton m_btnClipboard;
		CDlgButton m_btnDelete;
		CDlgButton m_btnClose;
		
	public : 		
		CDlgCombo m_cmbGraphs;

		CVwGraphs( ) {
			m_iID = IDD_GRAPHS;
			}
	
		~CVwGraphs() { 
			}

		//-- Initialize controls 
		void InitializeControls( ) { 
			m_picGraph.SetHandle( GetDlgItem(m_hDlg, IDC_PICGRAPHS), "Graph Image - click on items to edit");
			m_cmbGraphs.SetHandle( GetDlgItem(m_hDlg, IDC_CMBGRAPHS), "Select graph to display");
			m_btnClipboard.SetHandle( GetDlgItem(m_hDlg, IDC_BTNCLIPBOARD), "Copy graph image to clipboard");
			m_btnDelete.SetHandle( GetDlgItem(m_hDlg, IDC_BTNDELETE), "Remove the selected graph from the workspace");
			m_btnClose.SetHandle( GetDlgItem(m_hDlg, IDC_BTNCLOSE), "Close the graph view window");
			m_btnClipboard.SetButtonImage( GetInstance(), IDI_CLIPCOPY, 16, 16 );			
			}

		void AddGraph( CGraph * pGraph ) { 
			m_cmbGraphs.AddString( pGraph->m_cTitle.ConstBuffer() );
			m_cmbGraphs.SetCurSel( m_cmbGraphs.GetCount()-1 );
			InvalidateRect( m_hDlg, NULL, true );
			}
		
		void OnGraphSelect() { 
			InvalidateRect( m_hDlg, NULL, true );
			}

		void OnPaint() {
			if( g_aGraphs.Count()==0 ) 
				return;

			RECT rctPic;
			m_picGraph.GetControlBounds( &rctPic );
			
			//------------------------------------------------------ Update the display
			PAINTSTRUCT ps;
			HDC hdc = BeginPaint(m_hDlg, &ps);
			
			CGraph * pGraph = (CGraph *)g_aGraphs[ m_cmbGraphs.GetCurSel() ];
			if( pGraph != NULL ) 
				pGraph->Draw( hdc, rctPic );
			
			//-- debugging - show limits of draw area 	
			//MoveToEx( hdc, rctPic.left, rctPic.top, NULL );  LineTo( hdc, rctPic.right, rctPic.bottom);
			//MoveToEx( hdc, rctPic.right, rctPic.top, NULL );  LineTo( hdc, rctPic.left, rctPic.bottom);
	
			EndPaint(m_hDlg, &ps);
			}

		//----- if window is resized, adjust the graph display control 			
		void OnSize( WPARAM wParam, LPARAM * p ) { 
			//-- Desired size
			int iDlgWid=LOWORD(*p);
			int iDlgHgt=HIWORD(*p);

			//-- dialogue window info 
			RECT rcPar;
			GetBounds( &rcPar );
			MoveWindow( m_hDlg, 0, 0, iDlgWid, iDlgHgt, TRUE );
			
			//-- Resize picture control 
			RECT rctGraph;
			m_picGraph.GetControlBounds( &rctGraph );
			rctGraph.left--;							
			rctGraph.top--;							
			rctGraph.right = iDlgWid-(rcPar.right-rctGraph.right)+1;		// indent right same as left
			rctGraph.bottom = iDlgHgt-(rcPar.bottom-rctGraph.bottom);	// indent bottom same at left 
			MoveWindow( m_picGraph.GetHandle(), 
							rctGraph.left, 
							rctGraph.top, 
							rctGraph.right-rctGraph.left,
							rctGraph.bottom-rctGraph.top+1,	
							TRUE );

			}			

		//-- Limit display size to at least 600 x 275
		void OnMinMaxInfo( MINMAXINFO * p ) {  
			p->ptMinTrackSize.x = 600;
			p->ptMinTrackSize.y = 275;
			}

		//-----------------------------------------------------------------------------
		// Events and messages 
		void OnBtnDelete() { 
			int iSel = m_cmbGraphs.GetCurSel();
			if( iSel<0 || iSel>=m_cmbGraphs.GetCount() ) 
				return;
			m_cmbGraphs.DeleteString( iSel );
			g_aGraphs.DeleteItem( iSel );
			if( m_cmbGraphs.GetCount()>0 ) 
				m_cmbGraphs.SetCurSel(0);
			OnGraphSelect();
			}
			
		void OnBtnClose( ) { 
			DestroyWindow( GetParent( m_hDlg ) );
			}
			
		void OnBtnClipboard( ) { 
			CGraph * pGraph = (CGraph *)g_aGraphs[ m_cmbGraphs.GetCurSel() ];
			if( pGraph == NULL ) 
				return;

			CCopyImageProperties o;
			o.Set( 400, 400, true );
			if( GetCopyImageProperties( m_hDlg, &o ) ) { 
				//----- Create memory object and lock it 
				HGLOBAL hBits = GlobalAlloc( GHND, sizeof(BITMAPINFOHEADER)+o.m_iWidth*o.m_iHeight*3+1 );
				BYTE * pMem = (BYTE *)GlobalLock( hBits );

				//-- create bitmap DC
				HDC hdcWin = GetDC( m_hDlg );
				HDC hdcBmp = CreateCompatibleDC( hdcWin );

				//----- Prepare bitmap for Open GL rendering
				BITMAPINFO info;
				info.bmiHeader.biSize = sizeof(info.bmiHeader);
				info.bmiHeader.biWidth = o.m_iWidth;
				info.bmiHeader.biHeight = o.m_iHeight;
				info.bmiHeader.biPlanes = 1;
				info.bmiHeader.biBitCount = 24;
				info.bmiHeader.biCompression = BI_RGB;
				info.bmiHeader.biClrImportant = 0;
				info.bmiHeader.biClrUsed = 0;
				info.bmiHeader.biSizeImage = 0;
				info.bmiHeader.biXPelsPerMeter = 0;
				info.bmiHeader.biYPelsPerMeter = 0;
				info.bmiColors->rgbBlue=192;
				info.bmiColors->rgbGreen=192;
				info.bmiColors->rgbRed=192;

				//--
				BYTE * pBits;
				HBITMAP hBmp = CreateDIBSection( hdcBmp, &info, DIB_RGB_COLORS, (void **)(&pBits), NULL, 0);
				SelectObject( hdcBmp, hBmp );
				HFONT fntWas = (HFONT) SelectObject(  hdcBmp, GetStockObject(DEFAULT_GUI_FONT) );
				HBRUSH brWhite = CreateSolidBrush( RGB(255,255,255) );

				//-- Draw current graph to bitmap
				RECT rct;
				SetRect( &rct, 0, 0, o.m_iWidth, o.m_iHeight );
				FillRect( hdcBmp, &rct, brWhite );
				
				pGraph->Draw( hdcBmp, rct );

				memcpy( pMem, &info.bmiHeader, sizeof( BITMAPINFOHEADER ) );
				memcpy( pMem+sizeof( BITMAPINFOHEADER ), pBits, o.m_iWidth*o.m_iHeight*3 );

				SelectObject( hdcBmp, fntWas );
				DeleteObject( brWhite );
				DeleteObject( hBmp );
				DeleteDC( hdcBmp );
				ReleaseDC( m_hDlg, hdcWin );
				SetFocus( m_hDlg );

				GlobalUnlock( pMem );

				//----- Copy to clipboard
				if( OpenClipboard( m_hDlg ) ) { 
					EmptyClipboard();
					if( NULL==SetClipboardData( CF_DIB, hBits ) ) 
						GlobalFree( hBits );
					CloseClipboard( );
					}
				else 
					GlobalFree( hBits );
				}
			}
		
		void OnBtnProperties() { 
			CGraph * pGraph = (CGraph *)g_aGraphs[ m_cmbGraphs.GetCurSel() ];
			if( pGraph != NULL ) {
				GetGraphProperties( m_hDlg, pGraph );	
				InvalidateRect( m_hDlg, NULL, true );
				}
			}
			

	};

//-----------------------------------------------------------------------------
CVwGraphs * g_pVw;
		
void AddGraph( CGraph * pGraph ) { 
	g_aGraphs.AddItem( pGraph );
	if( g_pVw != NULL ) 
		g_pVw->AddGraph( pGraph );
	}

//-----------------------------------------------------------------------------
BOOL CALLBACK GraphsViewDlgProc(HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam) { 
	
	//-- create the CVwData to go with the dialog 
	if( iMsg == WM_INITDIALOG ) { 
		g_pVw = new CVwGraphs;
		g_pVw->m_hDlg=hDlg;
		g_pVw->InitializeControls();
		for( int iGraph=0; iGraph<g_aGraphs.Count(); ++iGraph ) 
			g_pVw->m_cmbGraphs.AddString( ((CGraph *)g_aGraphs[iGraph])->m_cTitle.ConstBuffer() );
		if( g_aGraphs.Count()>0 ) 
			g_pVw->m_cmbGraphs.SetCurSel(0);
			
		g_pVw->OnMDIActivate( NULL, GetParent(hDlg) );
		SetWindowLong( GetParent(hDlg), 0, (long) g_pVw ) ;
		return TRUE;
		}
	
	// Get the CVw pointer for this dialog 
	g_pVw = (CVwGraphs *) GetWindowLong(  GetParent( hDlg ), 0 );

	int wmId, wmEvent;
	switch( iMsg ) { 
		case WM_PAINT :
			g_pVw->OnPaint();
			return FALSE;

		case WM_COMMAND :
			wmId = LOWORD(wParam); 
			wmEvent = HIWORD(wParam); 
			switch( wmId ) { 
				//-- Selection List message handlers
				case IDC_CMBGRAPHS :
					if( wmEvent==CBN_SELCHANGE )
						g_pVw->OnGraphSelect();
					break;
					break;
					
				case IDC_BTNDELETE :
					g_pVw->OnBtnDelete();
					break;
				
				case IDC_BTNCLOSE :
					g_pVw->OnBtnClose();
					break;
					
				case IDC_BTNCLIPBOARD :  
					g_pVw->OnBtnClipboard();
					break;

				case IDC_BTNPROPERTIES : 
					g_pVw->OnBtnProperties();
					break;
					
				default : 
					return FALSE;
				}
			return TRUE;

		case WM_DESTROY :
			delete g_pVw;
			g_pVw=NULL;
			SetWindowLong( GetParent(hDlg), 0, 0 ) ;
			WindowListRemove( hDlg );
			return TRUE;

		default : 
			break;
		}
		
	return FALSE;
	} 



	
	
	
	
	
	
	
	
/*
Note : use of bitmap buffering is not neccesary here ? 

if desired - Add a bitmap to view class and update it as rqd, 
then paint will blit 

//TODO - current graph . Draw to bitmap dc( &o ) 


if( .Count()>0 ) { 

	CGraph * pGraph = ...;
	
	//-- Create bitmap for buffering display 
	int iWid = m_rctPreview.right-m_rctPreview.left-1;
	int iHgt = m_rctPreview.bottom-m_rctPreview.top-1;
	HDC hdcBitmap = CreateCompatibleDC( hdc );
	HBITMAP hBitmap = CreateCompatibleBitmap( hdc, iWid, iHgt );
	SelectObject( hdcBitmap, hBitmap );

	//-- Draw on Bitmap
	for( int i=0; i<iHgt && i<iWid; ++i ) { 
		SetPixel( hdcBitmap, i, i, RGB(255,0,0) );
		}
		
	//-- copy device context bitmap to display 
	if( ! BitBlt( hdc, 
			m_rctPreview.left+1, 
			m_rctPreview.top+1, 
			m_rctPreview.right-m_rctPreview.left-1, 
			m_rctPreview.bottom-m_rctPreview.top-1,	 
			hdcBitmap,							// Source bitmap 
			0,										// Source Left
			0,										// Source Top 
			SRCCOPY ) ) {
		DWORD dwGLE = GetLastError();
		++dwGLE;
		}
			
	DeleteObject( hBitmap );
	DeleteDC( hdcBitmap );
	}
*/
